from .._compat import IS_PY2 as IS_PY2
from .compile import compile_restricted_eval as compile_restricted_eval
from typing import Any

nltosp: Any
default_guarded_getattr = getattr

def default_guarded_getitem(ob, index): ...
def default_guarded_getiter(ob): ...

class RestrictionCapableEval:
    globals: Any
    rcode: Any
    ucode: Any
    used: Any
    __name__: Any
    expr: Any
    def __init__(self, expr) -> None: ...
    def prepRestrictedCode(self) -> None: ...
    def prepUnrestrictedCode(self) -> None: ...
    def eval(self, mapping): ...
    def __call__(self, **kw): ...
